/*
 *	This program reads any number of tokens from 
 *	the command line, and echoes the set of unique pathnames
 *	encountered thereupon.  Designed to be used with:
 *		o	GNU make $(shell) function
 *		o	find <path> -name <pattern> | xargs upaths
 *
 *	Suggested/intended use in a makefile: 
 *	$(PKG_INCLUDES) = $(shell /home/pkg -name '*.h' | xargs upaths -iI)
 *
 *	Derive your own uses as you see fit. 
 *
 *	20 Feb 2000 - Jim Tison (jtison@ntplx.net)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <search.h>
#include <getopt.h>

void	*root;
int	compare( const void *a, const void *b ) { 
  return strcmp( (char *)a, (char *)b );
}
int	intersperse;
char	ichar;


/*
 *	strippath(): 
 *		Return the path-only portion of a filename. If no path
 *		information present, return "."
 *
 *		<name> argument must not be pure const!! It is mutated. 
 */

char	*strippath( char * name ) { 
  char		*lsep;

  if( !name )
    return NULL;
  lsep = strrchr( name, '/' );		/* Find last path separator 		*/
  if( !lsep )
    return ".";			      	/* No path means CWD.			*/
  *lsep = 0;				/* Otherwise return a truncated string.	*/
  return name;
}

void	action( const void *nodep, const VISIT which, const int depth ) { 
  char		**datap = (char **)nodep;

  if( which == endorder || which == leaf ) { 
    if(intersperse)
      printf("-%c%s ", ichar, *datap);
    else
      printf("%s ", *datap);
    tdelete( nodep, &root, compare );
  }
}

int	main( int argc, char *argv[] ) { 
  int		i, c;
  char		*token;
  char		*junk = (char *)root;

  while( (c = getopt( argc, argv, "i:h" )) != EOF ) { 
    switch( c ) { 
    case 'i':
      ichar = *optarg;
      intersperse = 1;
      break;
    case 'h':
      printf("usage: %s [-i[char]] filename [... filename]\n", argv[0]);
    default:
      return 1;
    }
  }
  if( optind == argc )
    return 1;

  /*
   *	For each incoming path/filename, strip it down to just the path,
   *	and ensure it's unique.
   */
  for( i = optind; i < argc; i++ ) { 
    if( (token = strippath( argv[i] )) != NULL)
      if( tfind( token, &root, compare ))
	continue; 
      else
	tsearch( token, &root, compare );
  }
  /*
   *	Now walk the binary tree we built above and spit out the 
   *	results to stdout, one at a time
   */
  twalk( root, action );
  putchar('\n');
  return 0;
}						/* All done. Add final LF.	*/
      
    
      
